#!%PERL%
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.

use strict;
use utf8;

package Vhffs::Panel::Mail;

use DBI;
use POSIX qw(locale_h);
use HTML::Template;
use locale;
use Locale::gettext;
use CGI;
use CGI::Session;
use Vhffs::User;
use Vhffs::Main;
use Vhffs::Panel::Main;
use Vhffs::Services::Mail;

sub search {
    my ($main, $name) = @_;
    my $sql;
    my @params;
    my $mails = [];

    if( defined $name ) {
        $sql = 'SELECT m.domain, o.state, g.groupname FROM vhffs_mxdomain m INNER JOIN vhffs_object o ON o.object_id = m.object_id INNER JOIN vhffs_groups g ON g.gid = o.owner_gid WHERE m.domain LIKE ?';
        push(@params, '%'.lc($name).'%');
    } else {
        $sql = 'SELECT m.domain, o.state, g.groupname FROM vhffs_mxdomain m INNER JOIN vhffs_object o ON o.object_id = m.object_id INNER JOIN vhffs_groups g ON g.gid = o.owner_gid';
    }

    my $dbh = $main->get_db_object();
    my $sth = $dbh->prepare($sql) or return undef;

    return undef unless($sth->execute(@params));

    while(my $m = $sth->fetchrow_hashref('NAME_lc')) {
        $m->{state} = Vhffs::Functions::status_string_from_status_id($m->{state});
        push(@$mails, $m);
    }

    return $mails
}

=pod

=head2 getall_per_group

    $dns = Vhffs::Panel::Mail::getall_per_group($vhffs, $gid);

Returns an array of hashrefs (oid, displayname, active, state (localized string)) of all mail domains by
a given group.

=cut


sub getall_per_group
{
    my ( $main, $gid ) = @_;

    my $dbh = $main->get_db_object;
    my $sql = 'SELECT m.object_id AS oid, m.domain AS displayname, o.state FROM vhffs_mxdomain m INNER JOIN vhffs_object o ON m.object_id = o.object_id WHERE o.owner_gid = ? ORDER BY m.domain';
    my $sth = $dbh->prepare($sql) or return -1;
    $sth->execute($gid) or return -2;
    my $mails = [];
    while(my $m = $sth->fetchrow_hashref) {
        $m->{active} = ($m->{state} == Vhffs::Constants::ACTIVATED);
        $m->{refused} = ($m->{state} == Vhffs::Constants::VALIDATION_REFUSED);
        $m->{state} = Vhffs::Functions::status_string_from_status_id($m->{state});
        push @$mails, $m;
    }
    return $mails;
}

=pod

=head2 is_owned_by_group

    print "Group #$gid owns mail domain $domain\n" if(Vhffs::Panel::Mail::is_owned_by_group( $vhffs, $domain, $gid );

Returns true if a given mail domain is owned by a given group (GID), false otherwise.
C<$domain> is a string containing the mail domain name.

=cut

sub is_owned_by_group($$$) {
    my ($main, $domain, $gid) = @_;

    my $dbh = $main->get_db_object;
    my $sql = 'SELECT COUNT(*) FROM vhffs_mxdomain m INNER JOIN vhffs_object o ON o.object_id = m.object_id WHERE m.domain = ? AND o.owner_gid = ?';
    my $sth = $dbh->prepare( $sql );
    $sth->execute( $domain, $gid );
    my ($count) = $sth->fetchrow_array();
    return $count > 0;
}

sub create_mail
{
        my( $main , $domain , $description, $user , $group ) = @_;
        
        return undef if( ! defined $user );
        return undef if( ! defined $group );
        
        my $mail = Vhffs::Services::Mail::create($main, $domain, $description, $user, $group);

        return undef unless ( defined $mail );

        return undef if ( Vhffs::Acl::add_acl( $user , $mail , Vhffs::Constants::ACL_DELETE , $main ) < 0 );
        return undef if( Vhffs::Acl::add_acl( $group , $mail , Vhffs::Constants::ACL_VIEW , $main ) < 0 );

        return $mail;
}

1;
