/*
 *  VHFFSFSSYNC: Scalable file system replication over TCP
 *
 *  Copyright 2008  Sylvain Rochet <gradator@gradator.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */

#ifndef __linux__
#error This software is only running on Linux-based OS, bye!
#endif

#define _FILE_OFFSET_BITS 64
#define _ATFILE_SOURCE

#define DEBUG_NET 0
#define DEBUG_EVENTS 0

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <fcntl.h>
#include <dirent.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <glib.h>
#include <sys/select.h>
#include <signal.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <getopt.h>


/* -- network stuff -- */
// huge buffer size reduce syscalls
#define VHFFSFSSYNC_NET_RECV_CHUNK 65536
#define VHFFSFSSYNC_NET_RECV_OVERFLOW 104857600

typedef struct {
	char *data_buffer;
	ssize_t data_len;
	ssize_t data_cur;
} vhffsfssync_net_message;

typedef struct {
	int fd;
	struct sockaddr_in sockaddr;

	char *recvbuf;
	uint32_t recvbuf_begin;
	uint32_t recvbuf_end;

	FILE *chunk_file;
	ssize_t chunk_stilltoread;

	vhffsfssync_net_message **messages;
	uint32_t messages_begin;
	uint32_t messages_end;

	GHashTable *openfiles;

	double limitrate_speed;
	double limitrate_sleep;
	double limitrate_timeprev;
} vhffsfssync_conn;

// network protos
int vhffsfssync_net_send_data(vhffsfssync_conn *conn, char *data, ssize_t len);
void vhffsfssync_net_destroy_message(vhffsfssync_conn *conn, vhffsfssync_net_message *msg);
int vhffsfssync_net_send(vhffsfssync_conn *conn);

// events protos
int vhffsfssync_remove(char *pathname);
int vhffsfssync_mkdir(char *pathname, mode_t mode);
int vhffsfssync_checkfile(vhffsfssync_conn *conn, char *path, char *type, int long long size, int long long mtime);
int vhffsfssync_event(vhffsfssync_conn *conn, char *event);
int vhffsfssync_parse(vhffsfssync_conn *conn);

// misc
double vhffsfssync_time();
static void usage_exit(int ret_code, char *progname);

/* ------------------------------------------------------------ */

/* our events use \0 as a delimiter, a double \0 is the end of the event */
inline ssize_t vhffsfssync_net_event_len(char *data)  {
	ssize_t len = 0;
	do {
		len += strlen(data+len);  //glibc strlen() is incredibly fast, we use it as much as possible
		len++;
	} while( *(data+len) );
	len++;
	return len;
}

inline int vhffsfssync_net_send_event(vhffsfssync_conn *conn, char *data)  {
	return vhffsfssync_net_send_data(conn, data, vhffsfssync_net_event_len(data));
}

// !!!!!! the buffer is freed when the message has been sent, DON'T send static string and DON'T free() the data yourself
int vhffsfssync_net_send_data(vhffsfssync_conn *conn, char *data, ssize_t len)  {
	vhffsfssync_net_message *msg;
	if(!conn || !data || len <= 0) return -1;

	msg = malloc(sizeof(vhffsfssync_net_message));
	msg->data_buffer = data;
	msg->data_len = len;
	msg->data_cur = 0;
	if( !(conn->messages_end & 0x03FF) )
		conn->messages = realloc( conn->messages, (((conn->messages_end >>10) +1) <<10) * sizeof(vhffsfssync_net_message*) );
	conn->messages[conn->messages_end] = msg;
	conn->messages_end++;

	return 0;
}


void vhffsfssync_net_destroy_message(vhffsfssync_conn *conn, vhffsfssync_net_message *msg) {
	free(msg->data_buffer);
	free(msg);
}


int vhffsfssync_net_send(vhffsfssync_conn *conn)  {
	gboolean full = FALSE;

	if(!conn || conn->fd < 0) return -1;
#if DEBUG_NET
	printf("--------------------------------------------------\n");
	printf("conn: %d, to: %s\n", conn->fd, inet_ntoa(conn->sockaddr.sin_addr));
#endif
	while(!full && conn->messages)  {
		vhffsfssync_net_message *msg = conn->messages[conn->messages_begin];
		ssize_t written;
		ssize_t lentowrite;

#if DEBUG_NET
		printf("  buffer: %ld bytes, %ld already written\n", (long int)msg->data_len, (long int)msg->data_cur);
#endif
		/* try to empty the buffer */
		lentowrite = msg->data_len - msg->data_cur;
		written = write(conn->fd, msg->data_buffer + msg->data_cur, lentowrite);
		if(written < 0) {
			switch(errno)  {
				case EAGAIN:
				case EINTR:
#if DEBUG_NET
					printf("=====> EAGAIN on write()\n");
#endif
					full = TRUE;
					break;
				default:
					fprintf(stderr, "write() failed on socket %d: %s\n", conn->fd, strerror(errno));
					return -1;
			}
		}
		else {
			msg->data_cur += written;
#if DEBUG_NET
			printf("    %ld bytes written on %ld bytes\n", (long int)written, (long int)lentowrite);
#endif
			/* the buffer is not empty yet (but the SendQ into the kernel is) */
			if(written < lentowrite) {
				full = TRUE;
			}
			/* buffer is now empty */
			else {
				vhffsfssync_net_destroy_message(conn, msg);

				conn->messages_begin++;
				if(conn->messages_begin == conn->messages_end) {
					conn->messages_begin = 0;
					conn->messages_end = 0;
					free(conn->messages);
					conn->messages = NULL;
					break;
				}
				if(conn->messages_begin == 0x0400) {
					void *cur, *end;
					conn->messages_begin = 0;
					conn->messages_end -= 0x0400;
					cur = conn->messages;
					end = conn->messages + conn->messages_end;
					for( ; cur < end ; cur += 0x0400*sizeof(void*) )
						memcpy(cur, cur + 0x0400*sizeof(void*), 0x0400*sizeof(void*) );
					conn->messages = realloc( conn->messages, (((conn->messages_end >>10) +1) <<10) * sizeof(vhffsfssync_net_message*) );
				}
			}
		}
	}

	return 0;
}


/* ---------------------------------------- */
int vhffsfssync_remove(char *pathname)  {
	struct stat st;

	if(! lstat(pathname, &st) )  {

		if( S_ISDIR(st.st_mode) )  {
			DIR *d;
			struct dirent *dir;

			d = opendir(pathname);
			if(d) {
				while( (dir = readdir(d)) )  {
					if( strcmp(dir->d_name, ".") && strcmp(dir->d_name, "..") )  {
						char *path;
						char *sep = "/";
						if( pathname[strlen(pathname)-1] == '/' ) sep = "";
						path = g_strdup_printf("%s%s%s", pathname, sep, dir->d_name);
						vhffsfssync_remove(path);
						free(path);
					}
				}
				closedir(d);
				if( rmdir(pathname) < 0)  {
					fprintf(stderr, "cannot rmdir() '%s': %s\n", pathname, strerror(errno));
				}
			}
			else {
				fprintf(stderr, "cannot opendir() '%s': %s\n", pathname, strerror(errno));
			}
		}
		else {
			if( unlink(pathname) < 0)  {
				fprintf(stderr, "cannot unlink() '%s': %s\n", pathname, strerror(errno));
			}
		}
	}
	else {
		if(errno != ENOENT) {
			fprintf(stderr, "cannot lstat() '%s': %s\n", pathname, strerror(errno));
			return -1;
		}
	}

	return 0;
}


// the content of pathname is modified
int vhffsfssync_mkdir(char *pathname, mode_t mode)  {
	char *cur, *dirs[64];
	int i, fd, fd_, argc;

	argc = 0;
	cur = pathname;
	while(*cur != '\0' && argc < 64) {
		for( ; *cur != '/' && *cur != '\0' ; cur++ );
		dirs[argc++] = pathname;
		if( *cur == '/' ) {
			*cur = '\0';
			pathname = ++cur;
		}
	}

	fd = AT_FDCWD;
	for(i = 0 ; i < argc ; i++) {
		mkdirat(fd, dirs[i], 0755);
		fd_ = openat(fd, dirs[i], 0);
		if(fd >= 0) close(fd);
		fd = fd_;
		if(fd < 0)  {
			fprintf(stderr, "openat() failed on %s: %s\n", dirs[i], strerror(errno));
			break;
		}
	}
	if(fd >= 0) close(fd);
	if(i != argc) return -1;
	return 0;
}


int vhffsfssync_checkfile(vhffsfssync_conn *conn, char *path, char *type, int long long size, int long long mtime) {
	gboolean fetch = FALSE;
	struct stat st;

	//printf("%s\n", path);

	// file or link
	if(! lstat(path, &st) )  {
		if(! S_ISDIR(st.st_mode) ) {
			char *type_ = "unknown";

			if( S_ISREG(st.st_mode) )  {
				type_ = "file";
			}

			else if( S_ISLNK(st.st_mode) )  {
				type_ = "link";
			}
			/* we don't need other file types (chr, block, fifo, socket, ...) */

			if(strcmp(type, type_)) {
				vhffsfssync_remove(path);
				fetch = TRUE;
			}
			else if(st.st_size != size) {
				fetch = TRUE;
			}
			else if(st.st_mtime < mtime) {
				fetch = TRUE;
			}
		}
	}
	else {
		if(errno == ENOENT) {
			fetch = TRUE;
		}
		else {
			fprintf(stderr, "cannot lstat() '%s': %s\n", path, strerror(errno));
			return -1;
		}
	}

	if(fetch) {
		vhffsfssync_net_send_event(conn, g_strdup_printf("get%c%s%c", '\0', path, '\0') );
	}

	return 0;
}


int vhffsfssync_event(vhffsfssync_conn *conn, char *event)  {
	char *cur, **args = NULL;
	int argalloc = 0, argc = 0;
	int ret = 0;

	do {
		for(cur = event ; *cur++ != '\0' ; );

		if(argc >= argalloc)  {
			argalloc = ( (argc >>8) +1) <<8;
			args = realloc( args, argalloc * sizeof(char*) );
		}

		args[argc++] = event;
		event = cur;
	} while(*event);
	if(!argc) return -1;

#if DEBUG_EVENTS
	int i;
	for(i = 0 ; i < argc ; i++) {
		printf("%s ", args[i]);
	}
	printf("\n");
#endif

	if(!strcmp(args[0], "remove")) {
		char *pathname = args[1];
		vhffsfssync_remove(pathname);
	}
	else if(!strcmp(args[0], "create")) {
		char *pathname = args[1];
		int fd;

		fd = open(pathname, O_CREAT|O_WRONLY|O_TRUNC, 0644);
		if(fd >= 0) {
			close(fd);
		}
		else {
			fprintf(stderr, "open() failed on %s: %s\n", pathname, strerror(errno));
		}
	}
	else if(!strcmp(args[0], "open")) {
		char *pathname = args[1];
		int fd;

		if( !g_hash_table_lookup(conn->openfiles, pathname) ) {
			fd = open(pathname, O_CREAT|O_WRONLY|O_TRUNC, 0644);
			if(fd >= 0) {
				FILE *f = fdopen(fd, "w");
				if(f) {
					g_hash_table_insert(conn->openfiles, strdup(pathname), f);
				}
				else {
					fprintf(stderr, "fdopen() failed on %s: %s\n", pathname, strerror(errno));
					close(fd);
				}
			}
			else {
				fprintf(stderr, "open() failed on %s: %s\n", pathname, strerror(errno));
			}
		}
	}
	else if(!strcmp(args[0], "close")) {
		char *pathname = args[1];
		g_hash_table_remove(conn->openfiles, pathname);
	}
	else if(!strcmp(args[0], "mkdir")) {
		char *path = args[1];
		vhffsfssync_mkdir(path, 0755);
	}
	else if(!strcmp(args[0], "symlink")) {
		char *from = args[1];
		char *to = args[2];
		struct stat st;
		if(! lstat(from, &st) )  {
			vhffsfssync_remove(from);
		}
		if( symlink(to, from) )  {
			fprintf(stderr, "symlink() failed on %s -> %s: %s\n", from, to, strerror(errno));
		}
	}
	else if(!strcmp(args[0], "move")) {
		char *from = args[1];
		char *to = args[2];
		if( rename(from, to) ) {
			fprintf(stderr, "rename() failed from %s to %s: %s\n", from, to, strerror(errno));
			if( errno == ENOENT ) {
				vhffsfssync_net_send_event(conn, g_strdup_printf("get%c%s%c", '\0', to, '\0') );
			}
		}
	}
	else if(!strcmp(args[0], "write")) {
		char *pathname = args[1];
		off_t offset = atoll(args[2]);
		ssize_t size = atol(args[3]);
		int fd;

		//printf("FILE: %s %lld %d\n", pathname, offset, size);

		conn->chunk_stilltoread = size;

		conn->chunk_file = g_hash_table_lookup(conn->openfiles, pathname);
		if(!conn->chunk_file) {
			int flags;

			flags = O_CREAT|O_WRONLY;
			if(!size) flags |= O_TRUNC;   // just in case

			fd = open(pathname, flags, 0644);
			if(fd >= 0) {
				FILE *f = fdopen(fd, "w");
				if(f) {
					g_hash_table_insert(conn->openfiles, strdup(pathname), f);
					conn->chunk_file = f;
				}
				else {
					fprintf(stderr, "fdopen() failed on %s: %s\n", pathname, strerror(errno));
					close(fd);
				}
			}
			else {
				fprintf(stderr, "open() failed on %s: %s\n", pathname, strerror(errno));
			}
		}

		if(conn->chunk_file  &&  fseeko(conn->chunk_file, offset, SEEK_SET) < 0 ) {
			fprintf(stderr, "fseeko() on %lld failed on file %s: %s\n", (long long int)offset, pathname, strerror(errno));
			g_hash_table_remove(conn->openfiles, pathname);
			conn->chunk_file = NULL;
		}
	}
	else if(!strcmp(args[0], "ls")) {
		char *root, *root_;
		root = args[1];
		root_ = strdup(root);

		if(! vhffsfssync_mkdir(root, 0755) ) {
			int i;
			GHashTable *filesindex;
			DIR *d;

			// build an index with all files
			filesindex = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, NULL);

			// check each file
			for( i = 2 ; i < argc ; i+=4 ) {
				char *path;
				g_hash_table_insert(filesindex, args[i], args[i]);

				path = g_strdup_printf("%s/%s", root_, args[i]);
				vhffsfssync_checkfile(conn, path, args[i+1], atoll(args[i+2]), atoll(args[i+3]) );
				free(path);
			}

			// check for removed files
			d = opendir(root_);
			if(d) {
				struct dirent *dir;
				while( (dir = readdir(d)) )  {
					if(strcmp(dir->d_name, ".") && strcmp(dir->d_name, "..") )  {
						if(! g_hash_table_lookup(filesindex, dir->d_name) )  {
							char *path = g_strdup_printf("%s/%s", root_, dir->d_name);
							//printf("Deleting %s\n", path);
							vhffsfssync_remove(path);
							free(path);
						}
					}
				}
				closedir(d);
			}
			g_hash_table_destroy(filesindex);
		}
		free(root_);
	}
	else if(!strcmp(args[0], "time")) {
		time_t sec;
		struct timeval tv;
		gettimeofday(&tv, NULL);
		sec = atol(args[1]);
		if( abs(tv.tv_sec - sec) > 10 ) {
			fprintf(stderr, "The slave timestamp is not synchronous with the master timestamp\n");
			ret = -1;
		}
	}
	else if(!strcmp(args[0], "hello")) {
		// nice to meet you
	}
	else {
		fprintf(stderr, "Received unhandled event: %s\n", args[0]);
		ret = -1;
	}

	free(args);
	return ret;
}


int vhffsfssync_parse(vhffsfssync_conn *conn)  {
	char *cur, *end;

	//printf("Buffer %d, stilltoread: %d\n", conn->buf_len, conn->chunk_stilltoread);

	/* parse the buffer */
	cur = conn->recvbuf + conn->recvbuf_begin;
	end = conn->recvbuf + conn->recvbuf_end; //beware: end can be outside the buffer, you should NOT read *end

	while(cur < end)  {

		// text mode
		if(!conn->chunk_stilltoread)  {
			char *begin;
			// find "\0\0"
			for(begin = cur ; ( cur < end && *cur++ != '\0' ) || ( cur < end && *cur++ != '\0' ) ; );

			if( !*(cur-2) && !*(cur-1) ) {
				if( vhffsfssync_event(conn, begin) )
					return -1;
				conn->recvbuf_begin += (cur - begin);
				begin = cur;
			}

			if(cur == end)  {
				register uint32_t len = end - begin;
				//fprintf(stderr, "Not parsed %d, begin is %d, end id %d\n", len, conn->recvbuf_begin, conn->recvbuf_end);

				if(len) {
					if(len > VHFFSFSSYNC_NET_RECV_OVERFLOW)
						return -1;

					// copy the data that is not parsed to the begin of the buffer if the data don't overlap
					if(len <= conn->recvbuf_begin) {
						//printf("Realloc to %d bytes\n", len);
						memcpy(conn->recvbuf, begin, len);
						conn->recvbuf = realloc( conn->recvbuf , len );
						conn->recvbuf_begin = 0;
						conn->recvbuf_end = len;
					}
				}
				else {
					free(conn->recvbuf);
					conn->recvbuf = NULL;
					conn->recvbuf_begin = 0;
					conn->recvbuf_end = 0;
				}
				break;
			}
		}

		// binary mode (receiving a chunk)
		else {
			size_t canread = MIN(conn->chunk_stilltoread, end-cur);
			conn->chunk_stilltoread -= canread;
#if DEBUG_EVENTS
			printf("binary mode: read: %ld stilltoread: %ld\n", (long int)canread, (long int)conn->chunk_stilltoread);
#endif
			if(conn->chunk_file) {
				size_t len = fwrite(cur, 1, canread, conn->chunk_file);
#if DEBUG_EVENTS
				printf("  written: %ld\n", (long int)len);
#endif
				if(len != canread)  {
					fprintf(stderr, "fwrite() failed: %s\n", strerror(errno));
				}
				if(!conn->chunk_stilltoread) {
					conn->chunk_file = NULL;
				}
			}
			cur += canread;
			conn->recvbuf_begin += canread;

			// all the data have been read, resetting buffer
			if(cur == end) {
				free(conn->recvbuf);
				conn->recvbuf = NULL;
				conn->recvbuf_begin = 0;
				conn->recvbuf_end = 0;
				break;
			}
		}
	}

	return 0;
}


double vhffsfssync_time()  {
	struct timeval tv;
	gettimeofday(&tv, NULL);
	return tv.tv_sec + tv.tv_usec / 1e6;
}


static void usage_exit(int ret_code, char *progname)  {
	printf ("Usage: %s [OPTION]... HOST[:PORT] DIRECTORY\n"
		"Remote synchronous file-copying tool, this is the client (the slave)\n\n"
		"  -f, --foreground\tDo not daemonise the client, display errors on the console\n"
		"  -r, --limit-rate=kB/s\tLimit I/O bandwidth; kBytes per second\n"
		"  -h, --help\t\tDisplay this help and exit\n"
		"  -v, --version\t\tOutput version information and exit\n",
		progname);
	exit(ret_code);
}


int main(int argc, char *argv[])  {

	int flags;
	vhffsfssync_conn *conn;
	char *cur;

	int foreground = 0;
	char *host = NULL;
	int port = 4567;
	char *root = NULL;
	int limitrate = 0;

	struct option long_options[] = {
		{ "foreground", no_argument, NULL, 'f' },
		{ "limit-rate", required_argument, NULL, 'r' },
		{ "help", no_argument, NULL, 'h' },
		{ "version", no_argument, NULL, 'v' },
		{ 0, 0, 0, 0 }
	};

	while(1) {
		int option_index = 0, c;
		c = getopt_long(argc, argv, "fr:hv", long_options, &option_index);
		if(c == -1)
			break;

		switch(c) {
			case 'f':
				foreground = 1;
				break;

			case 'r':
				limitrate = atoi(optarg)*1000;
				break;

			case 'h':
				usage_exit(0, argv[0]);

			case 'v':
#ifdef VERSION
				fputs("vhffsfssync_slave " VERSION "\n", stdout);
#else
				fputs("vhffsfssync_slave\n", stdout);
#endif
				exit(0);

			case '?':
				/* `getopt_long' already printed an error message. */
				fprintf(stderr,"Try `%s --help' for more information.\n", argv[0]);
				exit(1);

			default:
				abort();
		}
	}

	if(optind != argc-2)
		usage_exit(1, argv[0]);

	host = argv[optind++];
	for(cur=host ; *cur ; cur++)  {
		if(*cur == ':')  {
			*cur = '\0';
			port = atoi(++cur);
			break;
		}
	}

	root = argv[optind++];

	if(!foreground) {
		close(STDIN_FILENO);
		close(STDOUT_FILENO);
		close(STDERR_FILENO);
		if(fork()) exit(0);
	}

	/* chdir() to the filesystem to write the data */
	if( chdir(root) < 0 ) {
		fprintf(stderr, "cannot chdir() to %s: %s\n", root, strerror(errno));
		return -1;
	}
	if( chroot(".") < 0 ) {
		fprintf(stderr, "cannot chroot() to %s: %s\n", root, strerror(errno));
		//return -1;
	}
	root = ".";

	/* -- network stuff -- */
	signal(SIGPIPE, SIG_IGN);
	conn = malloc(sizeof(vhffsfssync_conn));
	conn->fd = -1;
	conn->recvbuf = NULL;
	conn->recvbuf_begin = 0;
	conn->recvbuf_end = 0;
	conn->chunk_stilltoread = 0;
	conn->chunk_file = NULL;
	conn->messages = NULL;
	conn->messages_begin = 0;
	conn->messages_end = 0;
	conn->openfiles = NULL;
	conn->limitrate_speed = (double)limitrate;
	conn->limitrate_sleep = 0;
	conn->limitrate_timeprev = vhffsfssync_time();

	/* -- main loop -- */
	while(1)  {
		/* clean the previous connection */
		if(conn->fd >= 0) {
			shutdown(conn->fd, SHUT_RDWR);
			close(conn->fd);
		}
		conn->fd = -1;

		if(conn->openfiles) g_hash_table_destroy( conn->openfiles );
		conn->openfiles = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (gpointer)fclose);
		conn->chunk_file = NULL;

		if(conn->messages) {
			uint32_t i;
			for(i = conn->messages_begin ; i < conn->messages_end ; i++)
				vhffsfssync_net_destroy_message(conn,  conn->messages[i]);
			free(conn->messages);
		}
		conn->messages = NULL;
		conn->messages_begin = 0;
		conn->messages_end = 0;

		if(conn->recvbuf) free(conn->recvbuf);
		conn->recvbuf = NULL;
		conn->recvbuf_begin = 0;
		conn->recvbuf_end = 0;
		conn->chunk_stilltoread = 0;

		conn->limitrate_speed = (double)limitrate;
		conn->limitrate_sleep = 0;
		conn->limitrate_timeprev = vhffsfssync_time();

		/* connect */
		inet_aton(host, &conn->sockaddr.sin_addr);
		conn->sockaddr.sin_family = AF_INET;
		conn->sockaddr.sin_port = htons(port);

		conn->fd = socket(conn->sockaddr.sin_family, SOCK_STREAM, IPPROTO_IP);
		if(conn->fd < 0)  {
			fprintf(stderr, "socket() failed: %s\n", strerror(errno));
			return -1;
		}

		if( connect(conn->fd, (struct sockaddr*)&conn->sockaddr, sizeof(conn->sockaddr)) < 0 )  {

			fprintf(stderr, "connect() failed: %s\n", strerror(errno));
			goto disconnected;
		}

		/* set newfd to non-blocking */
		flags = fcntl(conn->fd, F_GETFL);
		if(flags >= 0) {
			flags |= O_NONBLOCK;
			fcntl(conn->fd, F_SETFL, flags);
		}

		/* connected */
		vhffsfssync_net_send_event(conn, g_strdup_printf("hello%c", '\0') );
		vhffsfssync_net_send_event(conn, g_strdup_printf("time%c", '\0') );
		vhffsfssync_net_send_event(conn, g_strdup_printf("fulltree%c", '\0') );

		/* -- the real main loop starts here -- */
		while(1)  {
			int max_fd = 0;
			fd_set readfs;
			fd_set writefs;
			//struct timeval tv;
			int ret;

			FD_ZERO(&readfs);
			FD_ZERO(&writefs);

			FD_SET(conn->fd, &readfs);
			if(conn->messages) FD_SET(conn->fd, &writefs);
			if(conn->fd > max_fd) max_fd = conn->fd;

			//tv.tv_sec = 3600;
			//tv.tv_usec = 0;
			ret = select(max_fd + 1, &readfs, &writefs, NULL, NULL);
			if(ret < 0)  {
				switch(errno)  {
					case EAGAIN:
					case EINTR:
						break;
					default:
						fprintf(stderr, "select() failed: %s\n", strerror(errno));
				}
			}
			if(ret > 0)  {
				/* data to read ?, give give ! */
				if(FD_ISSET(conn->fd, &readfs)  )  {
					ssize_t len;

					//fprintf(stdout, "Alloc %d bytes\n", conn->recvbuf_end + VHFFSFSSYNC_NET_RECV_CHUNK);
					conn->recvbuf = realloc( conn->recvbuf , conn->recvbuf_end + VHFFSFSSYNC_NET_RECV_CHUNK );
					len = read(conn->fd, conn->recvbuf+conn->recvbuf_end, VHFFSFSSYNC_NET_RECV_CHUNK);
					if(len < 0)  {
						switch(errno)  {
							case EAGAIN:
							case EINTR:
								break;
							default:
								fprintf(stdout, "read() failed on socket %d: %s\n", conn->fd, strerror(errno));
								goto disconnected;
						}
					}
					else if(len == 0) {
						goto disconnected;
					}
					else {
						//fprintf(stdout, "Read %d, buffer is %lu, begin is %lu, end is %lu\n", len, (unsigned long)conn->recvbuf, (unsigned long)conn->recvbuf_begin, (unsigned long)conn->recvbuf_end);
						conn->recvbuf_end += len;
						if( vhffsfssync_parse(conn) )
							goto disconnected;

						if(limitrate > 0)  {
							double current, delta, delta_t, a;

							// computing download rate
							current = vhffsfssync_time();
							delta_t = current - conn->limitrate_timeprev;
							conn->limitrate_timeprev = current;

							a = delta_t/2.0; // consider elapsed time, speed is roughly computed over 2 seconds
							if(a > 1.0) a = 1.0;
							else if(a < 0.0) a = 0.0; // might happens
							conn->limitrate_speed = (conn->limitrate_speed * (1.0-a)) + (((double)len / delta_t) * a);
							//printf("%.2f MB/s\n", conn->limitrate_speed/1000/1000 );

							delta = conn->limitrate_speed - (double)limitrate;  // delta is in bytes/s
							conn->limitrate_sleep += delta * 0.01;
							//printf("%.2f ms\n", conn->limitrate_sleep/1000 );
							if(conn->limitrate_sleep > 1000000.0) conn->limitrate_sleep = 1000000.0;
							if(conn->limitrate_sleep > 0.0) usleep((int)conn->limitrate_sleep);
						}
					}
				}

				/* data to send ?, send send ! */
				if(conn->messages && FD_ISSET(conn->fd, &writefs)  )  {
					if( vhffsfssync_net_send(conn) ) {
						goto disconnected;
					}
				}
			}
		}
// YES, I am evil, I am using goto and I don't care about !
disconnected:
#if DEBUG_NET
		printf("Byebye %s... (used fd %d)\n", inet_ntoa(conn->sockaddr.sin_addr), conn->fd);
#endif
		sleep(1);
	}

	return 0;
}
